import os
import re
import shutil
import music_tag
from .arguments import Arguments
from .settings import TMP_DIR, UNKNOWN


class FilesManager:
    def __init__(self, args: Arguments):
        self.args = args

    def manage(self):
        for filename in os.listdir(TMP_DIR):
            filepath = os.path.join(TMP_DIR, filename)
            match = re.search(r"^(\S+)_(.*)(\.\S+)$", filename)
            track_number = (
                int(match.groups()[0]) if match.groups()[0].isdigit() else None
            )
            song_name = match.groups()[1]
            ext = match.groups()[2]
            f = music_tag.load_file(filepath)
            f["title"] = song_name
            for artist in self.args.artists:
                f.append_tag("artist", artist)
            f["album"] = self.args.album
            if track_number:
                f["tracknumber"] = track_number
            f.save()
            dest_dir = "./{}/{}".format(
                " X ".join(self.args.artists) or UNKNOWN,
                self.args.album or UNKNOWN,
            )
            os.makedirs(dest_dir, exist_ok=True)
            shutil.move(filepath, os.path.join(dest_dir, song_name + ext))



import io
from contextlib import redirect_stderr
from .utils import printc, COLORS
from .arguments import Arguments
from .downloader import Downloader
from .files_manager import FilesManager


def main(argv=None):
    exit_code = 0
    print("hey")
    # f = io.StringIO()
    # try:
    #     args = Arguments(argv)
    #     args.check()
    #     with redirect_stderr(f):
    #         Downloader(args).download()
    # except SystemExit as e:
    #     stderr = f.getvalue()
    #     if e.code == 0:
    #         pass
    #     if "Private video" in stderr:
    #         printc(COLORS.WARNING, "Private tracks were not downloaded")
    #     else:
    #         printc(COLORS.FAIL, stderr)
    #         return e.code

    # FilesManager(args).manage()

    return exit_code



import shutil
from .arguments import Arguments
from yt_dlp import main as ytdlp
from .settings import TMP_DIR
from glob import glob
from os import remove


class Downloader:
    def __init__(self, args: Arguments):
        self.args = args

    def download(self):
        shutil.rmtree(TMP_DIR, ignore_errors=True)
        ytdlp_args = ["-x"]
        if self.args.split:
            ytdlp_args += [
                "-o",
                f"{TMP_DIR}to_delete.%(ext)s",
                "--split-chapters",
                "-o",
                f"chapter:{TMP_DIR}%(section_number)s_%(section_title)s.%(ext)s",
            ]
        else:
            ytdlp_args += ["-o", f"{TMP_DIR}%(playlist_index)s_%(title)s.%(ext)s"]

        ytdlp_args.append(self.args.url)
        try:
            ytdlp(ytdlp_args)
        except SystemExit as e:
            if e.code != 0:
                raise e
        if self.args.split:
            remove(glob(f"{TMP_DIR}to_delete.*")[0])



from argparse import ArgumentParser
from dataclasses import dataclass
from typing import List


@dataclass(init=False)
class Arguments:
    url: str
    artists: List[str]
    album: str
    split: bool

    def __init__(self, argv=None):
        parser = ArgumentParser(
            description="Command-line program to download music from YouTube and other websites."
        )
        parser.add_argument("url", help="URL of the song/album to download")
        parser.add_argument(
            "--artists",
            help="comma-separated artists names"
            ' like "Louis Armstrong,Ella Fitzgerald"',
            type=lambda s: [name for name in s.split(",")],
        )
        parser.add_argument("--album", help="album name")
        parser.add_argument(
            "--split",
            help="split song in multiple songs based on timestamps (youtube video description)",
            action="store_true",
        )

        args = parser.parse_args(argv)
        self.url = args.url
        self.artists = args.artists or []
        self.album = args.album
        self.split = args.split

    def check(self):
        """
        Checks the value and compatibility of arguments.
        """
        # Nothing todo yet
